﻿using ModuleTools.Attributes;
using ModuleTools.Interfaces;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Runtime.Serialization;

namespace DatabaseCore.Models.Department
{
	/// <summary>
	/// Класс, описывающий направление обучения кафедры
	/// </summary>
	[DataContract]
	[EntityDescription("EducationDirection", "Направление обучения кафедры")]
	[EntityDependency("Lecturer", "LecturerId", "Преподаватель, руководитель напарвления")]
	public class EducationDirection : BaseEntity, IEntitySecurityExtenstion<EducationDirection>
	{
		[DataMember]
        [Required(ErrorMessage = "required")]
        [MapConfiguration("Cipher")]
        public string Cipher { get; set; }

        [DataMember]
        [Required(ErrorMessage = "required")]
        [MapConfiguration("ShortName")]
        public string ShortName { get; set; }

        [DataMember]
        [Required(ErrorMessage = "required")]
        [MapConfiguration("Title")]
        public string Title { get; set; }

        [MapConfiguration("Profile")]
        public string Profile { get; set; }

        [DataMember]
        [Required(ErrorMessage = "required")]
        [MapConfiguration("Qualification")]
        public int Qualification { get; set; }

        [DataMember]
        [Required(ErrorMessage = "required")]
        [MapConfiguration("LecturerId")]
        public Guid LecturerId { get; set; }

        [MapConfiguration("Description")]
        public string Description { get; set; }

        //-------------------------------------------------------------------------

        public virtual Lecturer Lecturer { get; set; }

        //-------------------------------------------------------------------------

        [ForeignKey("EducationDirectionId")]
        public virtual List<AcademicPlan> AcademicPlans { get; set; }

        //-------------------------------------------------------------------------

        public EducationDirection SecurityCheck(EducationDirection entity, bool allowFullData) => entity;
	}
}